/*
----------------------
Be Sample Code License
----------------------

Copyright 1991-2001, Be Incorporated.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions, and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions, and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. The name of the author may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    
*/
#include <File.h>
#include <Directory.h>
#include <Entry.h>
#include <FindDirectory.h>
#include <Path.h>
#include "app.h"

BMessage *SettingsMsg;

App::App(const char *appsig):BApplication(appsig) {
	#if DEBUG
	PRINT(("App constructor\n"));
	#endif
	NetCenter = new NetCentral("The Core");
	LoadSettings();
	BRect r;
	SettingsMsg->FindRect("MainWindowFrame", &r);
	win = new Win(r, "dasPortal", B_TITLED_WINDOW, B_QUIT_ON_WINDOW_CLOSE, B_CURRENT_WORKSPACE);
}


App::~App() {
	#if DEBUG
	PRINT(("App destructor\n"));
	#endif

}


void App::LoadSettings() {
	BFile *file = new BFile("/boot/home/config/settings/dasPortal", B_READ_ONLY);
	SettingsMsg = new BMessage();
	if ((file->InitCheck() != B_OK)) {
		#if DEBUG
		PRINT(("Creating settings file.\n"));
		#endif
		file->SetTo("/boot/home/config/settings/dasPortal", B_CREATE_FILE | B_READ_WRITE);
	} else
		SettingsMsg->Unflatten(file);
	delete file;
	//Main Window Frame BRect
	if (!SettingsMsg->HasRect("MainWindowFrame")) {
		#if DEBUG
		PRINT(("Adding main window default location to the settings.\n"));
		#endif
		BRect r(100,100,400,200);
		SettingsMsg->AddRect("MainWindowFrame", r);
	}
	
	// send file attributes (other than mimetype)?
	if (!SettingsMsg->HasBool("SendAttribs")) {
		#if DEBUG
		PRINT(("Enabling sending of attributes in settings.\n"));
		#endif
		SettingsMsg->AddBool("SendAttribs", true); //you betcha!
	}
	
	//automatically accept all downloads?
	if (!SettingsMsg->HasBool("Auto-Accept-Downloads")) {
		#if DEBUG
		PRINT(("Enabling automatic acceptance of incoming files to the settings.\n"));
		#endif
		SettingsMsg->AddBool("Auto-Accept-Downloads", true);//why not?
	}
	
	//download directory
	if (!SettingsMsg->HasRef("DownloadDirectory")) {
		#if DEBUG
		PRINT(("Adding default downloads directory to settings.\n"));
		#endif
		BPath path, home;
		find_directory(B_USER_DIRECTORY, &home);
		path = home;
		path.Append("Downloads", false);
		BEntry ent(path.Path(), true);
		if (!ent.Exists()) {
			BDirectory dir(home.Path());
			dir.CreateDirectory(path.Path(), NULL);
		}
		entry_ref ref;
		ent.GetRef(&ref);
		SettingsMsg->AddRef("DownloadDirectory", &ref);
	}
	
	#if DEBUG
	PRINT(("App LoadSettings: SettingsMsg:\n"));
	SettingsMsg->PrintToStream();
	#endif
}


void App::SaveSettings() {
	BFile *file = new BFile("/boot/home/config/settings/dasPortal", B_WRITE_ONLY | B_ERASE_FILE);
	SettingsMsg->Flatten(file);
	file->Sync();
	delete file;
	delete SettingsMsg;
}


bool App::QuitRequested() {
	#if DEBUG
	PRINT(("App QuitRequested\n"));
	#endif
	BMessenger *msgr = new BMessenger(NULL, NetCenter, NULL);
	msgr->SendMessage(B_QUIT_REQUESTED);
	delete msgr;
	msgr=new BMessenger(NULL, win, NULL);
	msgr->SendMessage(B_QUIT_REQUESTED);
	delete msgr;
	SaveSettings();
	return true;
}


void App::ReadyToRun() {
	#if DEBUG
	PRINT(("App ReadyToRun\n"));
	#endif
	NetCenter->MainWindow = win;
	win->NetCenter = NetCenter;
	NetCenter->Run();
	win->Show();
}
